{% macro render_source(lines) -%}
<table class="code u-max-full-width">
  <tbody>
    <tr>
      <td class="lineno">
        <pre>
{%- for line in lines -%}
{{ line.number  }} {{ line|line_reason }}
{% endfor -%}
        </pre>
      </td>
      <td class="source">
        <pre>
{%- for line in lines -%}
          <span class="{{ line|line_status }}">{{ line.source|escape }}</span>
{%- endfor -%}
        </pre>
      </td>
    </tr>
  </tbody>
</table>
{%- endmacro %}
