<html>
  <head>
    <title>{{ title }}</title>
    <meta charset="UTF-8">
    <style>
{% include 'normalize.css' %}
{% include 'skeleton.css' %}
.hit {background-color: #EAFFEA}
.miss {background-color: #FFECEC}
.container .code {margin-left: 0}
pre {line-height: 1.3}
    </style>
  </head>
  <body>
    <div class="container">
      <h1>{{ title }}</h1>
      <table class="u-full-width">
        <thead>
          <tr>
            <th>Filename</th>
            <th>Stmts</th>
            <th>Miss</th>
            <th>Cover</th>
            <th>Missing</th>
          </tr>
        </thead>
        <tbody>
{%- for line in lines["Filename"] %}
          <tr>
            {%- if sources %}
            <td><a href="#{{ lines["Filename"][loop.index0] }}">{{ lines["Filename"][loop.index0] }}</a></td>
            {%- else %}
            <td>{{ lines["Filename"][loop.index0] }}</td>
            {%- endif %}
            <td>{{ lines["Stmts"][loop.index0] }}</td>
            <td>{{ lines["Miss"][loop.index0] }}</td>
            <td>{{ lines["Cover"][loop.index0] }}</td>
            <td>{{ lines["Missing"][loop.index0] }}</td>
          </tr>
{%- endfor %}
        </tbody>
        <tfoot>
          <tr>
            <td>{{ footer["Filename"] }}</td>
            <td>{{ footer["Stmts"] }}</td>
            <td>{{ footer["Miss"] }}</td>
            <td>{{ footer["Cover"] }}</td>
            <td>{{ footer["Missing"] }}</td>
          </tr>
        </tfoot>
      </table>
{%- if sources %}
{%- from 'macro.source.jinja2' import render_source -%}
{%- for filename, source in sources %}
<h4 id="{{ filename }}">{{ filename }}</h4>
{{ render_source(source) }}
{%- endfor %}
{% else %}
<p>{{ no_file_sources_message }}</p>
{%- endif %}
    </div>
  </body>
</html>
