<html>
  <head>
    <title>pycobertura report</title>
    <meta charset="UTF-8">
    <style>
{% include 'normalize.css' %}
{% include 'skeleton.css' %}
.red {color: red}
.green {color: green}
.hit {background-color: #EAFFEA}
.miss {background-color: #FFECEC}
pre {line-height: 1.2}
table.code pre {margin: 0;}
table.code td {padding: 0; border: 0;}
table.code td.lineno {text-align: right;}
table.code td.lineno pre {margin-right: 1rem;}
.legend dd {margin-left: 70px;}
.legend dt {float: left; width: 60px; text-align: center;}
.legend dt code {line-height: initial; border: none; background: none;}
    </style>
  </head>
  <body>
    <div class="container">
      <table class="u-full-width">
        <thead>
          <tr>
            <th>Filename</th>
            <th>Stmts</th>
            <th>Miss</th>
            <th>Cover</th>
{%- if show_source and show_missing %}
            <th>Missing</th>
{%- endif %}
          </tr>
        </thead>
        <tbody>
{%- for file_index in range(lines["Filename"]|length) %}
          <tr>
            <td><a href="#{{ lines["Filename"][file_index] }}">{{ lines["Filename"][file_index]}}</a></td>
            <td>{{ lines["Stmts"][file_index] }}</td>
            <td><span{% if lines["Miss"][file_index] | is_not_equal_to_dash %} class="{{ lines["Miss"][file_index] | misses_color }}"{% endif %}>{{ lines["Miss"][file_index] }}</span></td>
            <td>{{ lines["Cover"][file_index] }}</td>
{%- if show_source and show_missing %}
            <td>
{%- for missed_line in lines["Missing"][file_index] %}
            {%- if not loop.first %}, {% endif %}<span class="{{ missed_line | misses_color }}">{{ missed_line }}</span>
{%- endfor %}
            </td>
{%- endif %}
          </tr>
{%- endfor %}
        </tbody>
        <tfoot>
          <tr>
            <td>{{ footer["Filename"] }}</td>
            <td>{{ footer["Stmts"] }}</td>
            <td><span{% if footer["Miss"] | is_not_equal_to_dash %} class="{{ footer["Miss"] | misses_color }}"{% endif %}>{{ footer["Miss"] }}</span></td>
            <td>{{ footer["Cover"] }}</td>
{%- if show_source and show_missing %}
            <td></td>
{%- endif %}
          </tr>
        </tfoot>
      </table>
{%- if show_source %}
{%- from 'macro.source.jinja2' import render_source -%}
<div class="legend">
  <dl>
    <dt><code>code</code></dt><dd>coverage unchanged</dd>
    <dt class="hit"><code>code</code></dt><dd>coverage increased</dd>
    <dt class="miss"><code>code</code></dt><dd>coverage decreased</dd>
    <dt><code>+</code></dt><dd>line added or modified</dd>
  </dl>
</div>

{%- for filename, source_hunks in sources %}
<h4 id="{{ filename }}">{{ filename }}</h4>
{%- for hunk in source_hunks %}
{{ render_source(hunk) }}
{%- endfor %}
{%- endfor %}
{%- endif %}
    </div>
  </body>
</html>
